﻿#define HAS_ORACLE
#undef HAS_ORACLE

using System;
using System.Data;
#if HAS_ORACLE
using System.Data.OracleClient;
#endif
using System.Data.SqlClient;

namespace MoreTVPSamples
{
  class Program
  {
    static void Main(string[] args)
    {
    }

    private void Example1()
    {
      SqlConnection conn = new SqlConnection();
      // Set the connection string and open the connection

      DataSet ds = new DataSet();
      // Populate the dataset with an Order and OrderDetails table

      // Assumes conn is an open SqlConnection object and ds is
      // a DataSet with an Order and OrderDetails table
      using (conn)
      {
        // Create the command object to call the stored procedure
        SqlCommand cmd = new SqlCommand("uspInsertNewOrder", conn);
        cmd.CommandType = CommandType.StoredProcedure;

        // Create the parameter for passing the Order TVP
        SqlParameter headerParam = cmd.Parameters.AddWithValue
         ("@OrderTvp", ds.Tables["Order"]);

        // Create the parameter for passing the OrderDetails TVP
        SqlParameter detailsParam = cmd.Parameters.AddWithValue
         ("@OrderDetailsTvp", ds.Tables["OrderDetail"]);

        // Set the SqlDbType of the parameters to Structured
        headerParam.SqlDbType = SqlDbType.Structured;
        detailsParam.SqlDbType = SqlDbType.Structured;

        // Execute the stored procedure
        cmd.ExecuteNonQuery();
      }
    }

    private void Example2()
    {
      SqlConnection conn = new SqlConnection();
      // Set the connection string and open the connection

      DataSet ds = new DataSet();
      // Populate the dataset with a Category table

      // Define the INSERT INTO...SELECT statement to insert into Categories
      const string TSqlStatement =
       "INSERT INTO Categories (CategoryID, CategoryName)" +
       " SELECT nc.CategoryID, nc.CategoryName" +
       " FROM @NewCategoriesTvp AS nc";

	  // Assumes conn is an open SqlConnection object and ds is
      // a DataSet with a Category table
      using (conn)
      {
        // Set up the command object for the statement
        SqlCommand cmd = new SqlCommand(TSqlStatement, conn);

        // Add a TVP specifying the DataTable as the parameter value
        SqlParameter catParam = cmd.Parameters.AddWithValue
         ("@NewCategoriesTvp", ds.Tables["Category"]);

        catParam.SqlDbType = SqlDbType.Structured;
        catParam.TypeName = "dbo.CategoriesUdt";

        // Execute the command
        cmd.ExecuteNonQuery();
      }
    }

#if HAS_ORACLE
    private void Example3()
    {
      SqlConnection conn = new SqlConnection();
      // Set the connection string and open the connection

      OracleConnection oconn = new OracleConnection();
      // Set the connection string and open the connection

      // Set up command object to select from Oracle
      OracleCommand selCmd = new OracleCommand
       ("SELECT CategoryID, CategoryName FROM Categories;", oconn);

      // Execute the command and return the results in a connected
      // reader that will automatically close the connection when done
      OracleDataReader rdr = selCmd.ExecuteReader(CommandBehavior.CloseConnection);

      // Set up command object to insert to SQL Server
      SqlCommand insCmd = new SqlCommand("uspInsertCategories", conn);

      insCmd.CommandType = CommandType.StoredProcedure;

      // Add a TVP specifying the reader as the parameter value
      SqlParameter catParam = insCmd.Parameters.AddWithValue("@NewCategoriesTvp", rdr);

      catParam.SqlDbType = SqlDbType.Structured;

      // Execute the stored procedure
      insCmd.ExecuteNonQuery();
    }
#endif

  }
}
