USE master
GO

-- Create a file audit
CREATE SERVER AUDIT MyFileAudit
 TO FILE (FILEPATH='C:\Demo\SqlAudits')
GO


-- Enable it
ALTER SERVER AUDIT MyFileAudit
 WITH (STATE=ON)
GO


-- Rename the audit from MyFileAudit to SqlFileAudit, then rename it back
ALTER SERVER AUDIT MyFileAudit WITH (STATE=OFF)
ALTER SERVER AUDIT MyFileAudit MODIFY NAME = SqlFileAudit
ALTER SERVER AUDIT SqlFileAudit WITH (STATE=ON)
GO
ALTER SERVER AUDIT SqlFileAudit WITH (STATE=OFF)
ALTER SERVER AUDIT SqlFileAudit MODIFY NAME = MyFileAudit 
ALTER SERVER AUDIT MyFileAudit WITH (STATE=ON)
GO


-- Increase the asynchronous time-span for audit processing to one minute
ALTER SERVER AUDIT MyFileAudit WITH (STATE=OFF)
ALTER SERVER AUDIT MyFileAudit WITH (QUEUE_DELAY=60000)
ALTER SERVER AUDIT MyFileAudit WITH (STATE=ON)
GO


-- Change the on-failure setting to SHUTDOWN
ALTER SERVER AUDIT MyFileAudit WITH (STATE=OFF)
ALTER SERVER AUDIT MyFileAudit WITH (ON_FAILURE=SHUTDOWN)
ALTER SERVER AUDIT MyFileAudit WITH (STATE=ON)
GO


-- Monitor server for all logins and record to file system
CREATE SERVER AUDIT SPECIFICATION CaptureLoginsToFile
 FOR SERVER AUDIT MyFileAudit
  ADD (FAILED_LOGIN_GROUP),
  ADD (SUCCESSFUL_LOGIN_GROUP)
 WITH (STATE=ON)
GO


-- Also monitor changed passwords, and stop monitoring successful logins
ALTER SERVER AUDIT SPECIFICATION CaptureLoginsToFile WITH (STATE=OFF)
ALTER SERVER AUDIT SPECIFICATION CaptureLoginsToFile
 ADD (LOGIN_CHANGE_PASSWORD_GROUP),
 DROP (SUCCESSFUL_LOGIN_GROUP)
ALTER SERVER AUDIT SPECIFICATION CaptureLoginsToFile WITH (STATE=ON)
GO


-- Create an event log audit
USE master
GO

CREATE SERVER AUDIT MyEventLogAudit TO APPLICATION_LOG
ALTER SERVER AUDIT MyEventLogAudit WITH (STATE=ON)

-- Create a new database
IF EXISTS(SELECT name FROM sys.databases WHERE name = 'MyDB')
 DROP DATABASE MyDB 
GO
CREATE DATABASE MyDB
GO
USE MyDB
GO

-- Monitor database for all DML actions by all users to the event log
CREATE DATABASE AUDIT SPECIFICATION CaptureDbActionsToEventLog
 FOR SERVER AUDIT MyEventLogAudit
  ADD (DATABASE_OBJECT_CHANGE_GROUP),
  ADD (SELECT, INSERT, UPDATE, DELETE
        ON SCHEMA::dbo
        BY public)
 WITH (STATE=ON)


-- Using sys.fn_get_audit_file
SELECT event_time, database_name, object_name, statement 
 FROM sys.fn_get_audit_file('C:\Demo\SqlAudits\*.sqlaudit', default, default);
GO


-- Using the audit catalog views
SELECT * FROM sys.server_file_audits 
SELECT * FROM sys.server_audit_specifications 
SELECT * FROM sys.server_audit_specification_details 
SELECT * FROM sys.database_audit_specifications 
SELECT * FROM sys.database_audit_specification_details 
SELECT * FROM sys.dm_server_audit_status 
SELECT * FROM sys.dm_audit_actions 
SELECT * FROM sys.dm_audit_class_type_map 

