----------------------------------- Full-Text Search -----------------------------------

-- char, varchar, nchar, nvarchar, text, ntext, image, xml, varbinary, and varbinary(max)

-- Show supported languages
SELECT * FROM sys.fulltext_languages ORDER BY name

-- Show system-defined stop list (noise words)
SELECT * FROM sys.fulltext_system_stopwords

-- Show user stop lists
SELECT * FROM sys.fulltext_stoplists
SELECT * FROM sys.fulltext_stopwords


-- Setting up FTS

USE AdventureWorks2008R2
GO

-- Create the FTS catalog
CREATE FULLTEXT CATALOG DocumentCatalog
GO

-- Create the FTS Index
CREATE FULLTEXT INDEX ON Production.Document
 (Document TYPE COLUMN FileExtension LANGUAGE 2057)
 KEY INDEX PK_Document_DocumentNode
 ON DocumentCatalog
 WITH CHANGE_TRACKING AUTO
GO

-- Check Status
SELECT FULLTEXTCATALOGPROPERTY('DocumentCatalog', 'PopulateStatus')

-- Show all the catalogs
SELECT * FROM sys.fulltext_catalogs

-- Show all the indexes
SELECT
  CatalogName = c.name,
  SchemaName = s.name,
  TableName = ao.name,
  ColumnName = ac.name,
  Lang = ic.language_id,
  UpdateMode = i.change_tracking_state_desc
 FROM
  sys.fulltext_indexes AS i
  INNER JOIN sys.fulltext_catalogs AS c ON c.fulltext_catalog_id = i.fulltext_catalog_id
  INNER JOIN sys.fulltext_index_columns AS ic ON i.object_id = ic.object_id
  INNER JOIN sys.all_columns AS ac ON ac.column_id = ic.column_id AND ac.object_id = ic.object_id
  INNER JOIN sys.all_objects AS ao ON ao.object_id = ac.object_id
  INNER JOIN sys.schemas AS s ON s.schema_id = ao.schema_id
  
-- Query using CONTAINS with <simple-term>
SELECT * FROM Production.Document WHERE CONTAINS(Document, 'Mountain')
SELECT * FROM Production.Document WHERE FREETEXT(Document, 'vital safetey components')


GO
----------------------------------------------------------------------------------------------------

CREATE DATABASE ChapterLibrary
 ON PRIMARY
  (NAME = ChapterLibrary_data,
   FILENAME = 'C:\Demo\PhotoLibrary\ChapterLibrary_data.mdf'),
 FILEGROUP FileStreamGroup CONTAINS FILESTREAM
  (NAME = ChapterLibrary_blobs,
   FILENAME = 'C:\Demo\PhotoLibrary\PhotoFiles')
 LOG ON
  (NAME = ChapterLibrary_log,
   FILENAME = 'C:\Demo\PhotoLibrary\ChapterLibrary_log.ldf')
 WITH FILESTREAM
  (DIRECTORY_NAME='ChapterLibrary',
   NON_TRANSACTED_ACCESS=FULL)
GO

USE ChapterLibrary
GO

CREATE TABLE DocFiles AS FileTable
GO

-- Copy entire Chapters folder into FileTable

-- Delete all files except .doc files
DELETE FROM DocFiles WHERE file_type <> 'doc' AND is_directory = 0


-- Create the FTS catalog
CREATE FULLTEXT CATALOG DocumentCatalog
GO

-- Create the FTS Index
CREATE FULLTEXT INDEX ON DocFiles(
 file_stream TYPE COLUMN file_type LANGUAGE 2057,
 name LANGUAGE 2057)
  KEY INDEX PK__DocFiles__5A5B77D574B53615
  ON DocumentCatalog
  WITH CHANGE_TRACKING AUTO
GO

-- Check Status
SELECT FULLTEXTCATALOGPROPERTY('DocumentCatalog', 'PopulateStatus')

-- Show all the catalogs
SELECT * FROM sys.fulltext_catalogs

-- Show all the indexes
SELECT
  CatalogName = c.name,
  IsEnabled = i.is_enabled,
  SchemaName = s.name,
  TableName = ao.name,
  ColumnName = ac.name,
  Lang = ic.language_id,
  UpdateMode = i.change_tracking_state_desc
 FROM
  sys.fulltext_indexes AS i
  INNER JOIN sys.fulltext_catalogs AS c ON c.fulltext_catalog_id = i.fulltext_catalog_id
  INNER JOIN sys.fulltext_index_columns AS ic ON i.object_id = ic.object_id
  INNER JOIN sys.all_columns AS ac ON ac.column_id = ic.column_id AND ac.object_id = ic.object_id
  INNER JOIN sys.all_objects AS ao ON ao.object_id = ac.object_id
  INNER JOIN sys.schemas AS s ON s.schema_id = ao.schema_id

SELECT * FROM DocFiles WHERE FREETEXT(file_stream, 'hierarchyid')
SELECT * FROM DocFiles WHERE FREETEXT(file_stream, 'SampleDb')

SELECT * FROM sys.fulltext_document_types
EXEC sp_fulltext_service @action='load_os_resources', @value=1


----------------------------------- Semantic Search -----------------------------------

-- Check if Full-Text Search is installed
SELECT SERVERPROPERTY('IsFullTextInstalled')

-- Semantic Search requires the Semantic Language Statistics Database
SELECT * FROM sys.fulltext_semantic_language_statistics_database

-- To install the SLSD:
--  Run SemanticLanguageDatabase.msi (no uninstall)
--  Move the .mdf/.ldf files to standard location
--  Attach and register as follows
CREATE DATABASE semanticsdb
 ON (FILENAME='C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\semanticsdb.mdf')
 FOR ATTACH
GO

EXEC sp_fulltext_semantic_register_language_statistics_db @dbname = 'semanticsdb'
GO

-- Show supported languages
SELECT * FROM sys.fulltext_semantic_languages

