﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;

namespace DemoDataSets
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
    }

    private void btnGenericDataSet_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string SelectTSql = "SELECT * FROM Customer";

      const string UpdateTSql = @"
    UPDATE Customer
      SET FirstName = @FirstName, LastName = @LastName, UpdatedAt = SYSDATETIME()
      WHERE CustomerId = @CustomerId";

      var ds = new DataSet();

      using (var cn = new SqlConnection())
      {
        cn.ConnectionString = ConnStr;
        using (var cm = new SqlCommand())
        {
          cm.Connection = cn;
          cm.CommandText = SelectTSql;

          using (var da = new SqlDataAdapter())
          {
            da.SelectCommand = cm;
            da.Fill(ds);
          }
        }
      }

      // The self-tracking DataSet can now be serialized across the tiers

      var customerTable = ds.Tables[0];
      if (customerTable.Rows.Count == 0)
      {
        MessageBox.Show("There are no customers");
        return;
      }

      var firstCustomerRow = customerTable.Rows[0];

      // Type casting required
      var customerId = (long)firstCustomerRow["CustomerId"];
      var updatedAt = (DateTime)firstCustomerRow["UpdatedAt"];

      MessageBox.Show(string.Format("Customer {0} was updated at {1}",
        customerId, updatedAt));

      firstCustomerRow["FirstName"] = "Brian";
      firstCustomerRow["LastName"] = "Perry";

      using (var cn = new SqlConnection())
      {
        cn.ConnectionString = ConnStr;
        using (var cm = new SqlCommand())
        {
          cm.Connection = cn;
          cm.CommandText = UpdateTSql;
          cm.Parameters.Add("@FirstName", SqlDbType.VarChar, 50, "FirstName");
          cm.Parameters.Add("@LastName", SqlDbType.VarChar, 50, "LastName");
          cm.Parameters.Add("@CustomerId", SqlDbType.BigInt, -1, "CustomerId");

          using (var da = new SqlDataAdapter())
          {
            da.UpdateCommand = cm;
            da.Update(customerTable);
          }
        }
      }

    }

    private void btnTypedDataSet_Click(object sender, EventArgs e)
    {
      var ds = new SampleDS();
      using (var da = new SampleDSTableAdapters.CustomerTableAdapter())
      {
        da.Fill(ds.Customer);
      }

      using (var da = new SampleDSTableAdapters.OrderHeaderTableAdapter())
      {
        da.Fill(ds.OrderHeader);
      }

      // The self-tracking DataSet can now be serialized across the tiers

      if (ds.Customer.Count == 0)
      {
        MessageBox.Show("There are no customers");
        return;
      }

      var firstCustomerRow = ds.Customer[0];

      // No type casting required
      var customerId = firstCustomerRow.CustomerId;
      var updatedAt = firstCustomerRow.UpdatedAt;

      var orderRows = firstCustomerRow.GetOrderHeaderRows();

      MessageBox.Show(
        string.Format("Customer {0} was updated at {1} and has {2} order(s)",
          customerId, updatedAt, orderRows.Length));

      // Change the customer's name
      firstCustomerRow.FirstName = "David";
      firstCustomerRow.LastName = "Jones";

      // Add a customer order
      ds.OrderHeader.AddOrderHeaderRow(firstCustomerRow,
        "Regular Mail", "Open", string.Empty, DateTime.Now, DateTime.Now);

      using (var tam = new SampleDSTableAdapters.TableAdapterManager())
      {
        tam.CustomerTableAdapter =
          new SampleDSTableAdapters.CustomerTableAdapter();

        tam.OrderHeaderTableAdapter =
          new SampleDSTableAdapters.OrderHeaderTableAdapter();

        tam.UpdateAll(ds);
      }
    }

    private void btnLinqToGenericDataSet_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string SelectTSql = "SELECT * FROM Customer";

      var ds = new DataSet();

      using (var cn = new SqlConnection())
      {
        cn.ConnectionString = ConnStr;
        using (var cm = new SqlCommand())
        {
          cm.Connection = cn;
          cm.CommandText = SelectTSql;

          using (var da = new SqlDataAdapter())
          {
            da.SelectCommand = cm;
            da.Fill(ds);
          }
        }
      }

      var query =
        from cust in ds.Tables[0].AsEnumerable()
        where !cust.Field<string>("LastName").StartsWith("A")
        orderby cust.Field<string>("LastName") descending
        select cust;

      var dv = query.AsDataView();

      MessageBox.Show(string.Format(
        "Filtered {0} of {1} customers", dv.Count, ds.Tables[0].Rows.Count));
    }

    private void btnLinqToTypedDataSet_Click(object sender, EventArgs e)
    {
      var ds = new SampleDS();
      using (var da = new SampleDSTableAdapters.CustomerTableAdapter())
      {
        da.Fill(ds.Customer);
      }

      var query =
        from cust in ds.Customer
        where !cust.LastName.StartsWith("A")
        orderby cust.LastName descending
        select cust;

      var dv = query.AsDataView();

      MessageBox.Show(string.Format(
        "Filtered {0} of {1} customers", dv.Count, ds.Customer.Count));
    }

  }
}
