﻿using System;
using System.Collections.Generic;
using System.Data.Services.Client;
using System.Linq;
using System.Windows.Forms;

using DemoRestClientDataEntry.CustomerDataService;

namespace DemoRestClientDataEntry
{
  public partial class Form1 : Form
  {
    private SampleDbEntities _context;

    private List<Customer> _customers;
    private Customer _currentCustomer;

    private BindingSource _customersBindingSource;
    private BindingSource _ordersBindingSource;

    public Form1()
    {
      InitializeComponent();
    }

    private void btnLoad_Click(object sender, EventArgs e)
    {
      this.CreateNewContext();

      var query =
        from customer in this._context.Customers.Expand("OrderHeaders")
        select customer;

      this._customers = query.ToList();

      this._customersBindingSource = new BindingSource(this._customers, null);
      this.BindGrid(this.grdCustomers, this._customersBindingSource);

      this.grdOrders.DataSource = null;

      this.ShowCustomerOrders();
    }

    private void CreateNewContext()
    {
      var serviceUri = new Uri("http://localhost.:1055/CustomerDataService.svc");
      this._context = new SampleDbEntities(serviceUri);
    }

    private void grdCustomers_SelectionChanged(object sender, EventArgs e)
    {
      this.ShowCustomerOrders();
    }

    private void ShowCustomerOrders()
    {
      if (this.grdCustomers.SelectedRows.Count == 1)
      {
        this._currentCustomer =
         (Customer)this.grdCustomers.SelectedRows[0].DataBoundItem;

        this._ordersBindingSource =
         new BindingSource(this._currentCustomer.OrderHeaders, null);

        this.BindGrid(this.grdOrders, this._ordersBindingSource);
      }
    }

    private void btnNewCustomer_Click(object sender, EventArgs e)
    {
      this._customersBindingSource.AddNew();
    }

    private void grdCustomers_CellValueChanged
     (object sender, DataGridViewCellEventArgs e)
    {
      var customer = (Customer)this.grdCustomers.SelectedRows[0].DataBoundItem;

      var changeInfo =
       this._context.Entities.SingleOrDefault(ed => ed.Entity == customer);

      if (changeInfo == null)
      {
        this._context.AddToCustomers(customer);
      }
      else if (changeInfo.State == EntityStates.Unchanged)
      {
        this._context.UpdateObject(customer);
      }
    }

    private void btnDeleteCustomer_Click(object sender, EventArgs e)
    {
      var customer = (Customer)this.grdCustomers.SelectedRows[0].DataBoundItem;

      this._customersBindingSource.RemoveCurrent();
      this.grdOrders.DataSource = null;

      var changeInfo =
       this._context.Entities.SingleOrDefault(ed => ed.Entity == customer);

      if (changeInfo != null)
      {
        foreach (var orderHeader in customer.OrderHeaders)
        {
          this._context.DeleteObject(orderHeader);
        }

        this._context.DeleteObject(customer);
      }
    }

    private void btnNewOrder_Click(object sender, EventArgs e)
    {
      if (this.grdOrders.Columns.Count == 0)
      {
        this.ShowCustomerOrders();
        this.grdCustomers.EndEdit();
      }

      var orderHeader = (OrderHeader)this._ordersBindingSource.AddNew();

      orderHeader.CustomerId = this._currentCustomer.CustomerId;
      orderHeader.Customer = this._currentCustomer;
    }

    private void grdOrders_CellValueChanged
     (object sender, DataGridViewCellEventArgs e)
    {
      var orderHeader = (OrderHeader)this.grdOrders.SelectedRows[0].DataBoundItem;

      var changeInfo =
       this._context.Entities.SingleOrDefault(ci => ci.Entity == orderHeader);

      if (changeInfo == null)
      {
        this._context.AddToOrderHeaders(orderHeader);
      }
      else if (changeInfo.State == EntityStates.Unchanged)
      {
        this._context.UpdateObject(orderHeader);
      }
    }

    private void btnDeleteOrder_Click(object sender, EventArgs e)
    {
      var orderHeader = (OrderHeader)this.grdOrders.SelectedRows[0].DataBoundItem;

      this._ordersBindingSource.RemoveCurrent();
      this.grdOrders.DataSource = null;

      var changeInfo =
       this._context.Entities.SingleOrDefault(ed => ed.Entity == orderHeader);

      if (changeInfo != null)
      {
        this._context.DeleteObject(orderHeader);
      }
    }

    private void btnSave_Click(object sender, EventArgs e)
    {
      this.grdCustomers.EndEdit();
      this.grdOrders.EndEdit();

      try
      {
        this._context.SaveChanges(SaveChangesOptions.Batch);
      }
      catch (Exception ex)
      {
        MessageBox.Show
         (ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }

      this.grdCustomers.Refresh();
      this.grdOrders.Refresh();
    }

    private void BindGrid(DataGridView grd, BindingSource bs)
    {
      var readOnlyColumns = new string[] { "CustomerId", "OrderHeaderId", "CreatedAt", "UpdatedAt" };

      var hiddenColumns = new string[] { "Customer" };

      grd.DataSource = bs;
      foreach (DataGridViewColumn col in grd.Columns)
      {
        col.ReadOnly = readOnlyColumns.Contains(col.HeaderText);
        col.Visible = !hiddenColumns.Contains(col.HeaderText);
      }
      grd.AutoResizeColumns();
    }

  }
}
