﻿using System;
using System.ServiceModel.DomainServices.Client;
using System.Text;
using System.Windows;
using System.Windows.Controls;

using DemoSL.Web;

namespace DemoSL
{
  public partial class MainPage : UserControl
  {
    private SampleRiaContext _ctx = new SampleRiaContext();

    public MainPage()
    {
      InitializeComponent();
    }

    private void btnLoad_Click(object sender, RoutedEventArgs e)
    {
      this.btnLoad.IsEnabled = false;

      var lo = this._ctx.Load(
        this._ctx.GetCustomersQuery(),
        this.LoadOperationComplete,
        null);

      this.grdCustomers.ItemsSource = lo.Entities;
    }

    private void LoadOperationComplete(LoadOperation<Customer> lo)
    {
      if (lo.HasError)
      {
        MessageBox.Show(lo.Error.Message, "Load failed", MessageBoxButton.OK);
        lo.MarkErrorAsHandled();
      }
      this.btnLoad.IsEnabled = true;
    }

    private void btnSave_Click(object sender, RoutedEventArgs e)
    {
      this.btnSave.IsEnabled = false;

      this._ctx.SubmitChanges(this.SaveOperationCompleted, null);
    }

    private void SaveOperationCompleted(SubmitOperation so)
    {
      this.btnSave.IsEnabled = true;
      if (!so.HasError)
      {
        return;
      }

      var sb = new StringBuilder();
      sb.AppendLine(so.Error.Message);

      foreach (var badEntity in so.EntitiesInError)
      {
        sb.AppendFormat(string.Format("Entity {0} Error", badEntity.GetIdentity()));
        sb.AppendLine();
        foreach (var ve in badEntity.ValidationErrors)
        {
          sb.AppendLine(" " + ve.ErrorMessage);
        }
      }

      MessageBox.Show(sb.ToString(), "Submit failed", MessageBoxButton.OK);

      so.MarkErrorAsHandled();
    }

    private void grdCustomers_SelectionChanged
     (object sender, SelectionChangedEventArgs e)
    {
      var cust = this.grdCustomers.SelectedItem as Customer;
      if (cust != null)
      {
        this.grdOrders.ItemsSource = cust.OrderHeaders;
      }
    }

    private void btnNewCustomer_Click(object sender, RoutedEventArgs e)
    {
      var cust = new Customer();
      this._ctx.Customers.Add(cust);
      this.grdCustomers.ItemsSource = this._ctx.Customers;
    }

    private void btnDeleteCustomer_Click(object sender, RoutedEventArgs e)
    {
      var cust = this.grdCustomers.SelectedItem as Customer;
      if (cust != null)
      {
        this._ctx.Customers.Remove(cust);
        this.grdCustomers.ItemsSource = this._ctx.Customers;
      }
    }

    private void btnNewOrder_Click(object sender, RoutedEventArgs e)
    {
      var cust = this.grdCustomers.SelectedItem as Customer;
      if (cust != null)
      {
        var order = new OrderHeader();
        cust.OrderHeaders.Add(order);
        this.grdOrders.ItemsSource = cust.OrderHeaders;
      }
    }

    private void btnDeleteOrder_Click(object sender, RoutedEventArgs e)
    {
      var cust = this.grdCustomers.SelectedItem as Customer;
      if (cust != null)
      {
        var order = new OrderHeader();
        cust.OrderHeaders.Remove(order);
        this.grdOrders.ItemsSource = cust.OrderHeaders;
      }
    }

  }
}
